################################################################################
# File:             myDataDepth.R
# Created by:       Pierre Lafaye De Micheaux
# First released:   06.11.2018
# Last revised:     06.11.2018
# 
# Contains R-script for check of the stability of finding the deepest curve.
# 
# For a detailed description see:
#   Lafaye De Micheaux, P., Mozharovskyi, P. and Vimond, M. (2018). Depth for 
#     curve data and applications.
################################################################################

#### Check the stability of finding the deepest curve
# using our new function search.for.deepest.curves()
# We use only on the file 101_1_dwi.voxelcoordsL.txt
# hoping that this should work similarly on all other files

rm(list = ls())
require("curveDepth")
require("AnalyzeFMRI")
source("../DTI.R")
fibers.Left <- f.read.fibers(fibers.file = "101_1_dwi.voxelcoordsL.txt")
fibers.Left.coord <- fibers.Left$fibers.coordinates
fibers.Left.nb <- fibers.Left$nbfibers
fibers.Left.min.nbpoints <- min(fibers.Left$vect.of.nbpts.per.fiber)

n <- fibers.Left.nb # Number of curves taken from each side of the brain. 
# It is 1,000 for this specific example

leftFibers <- list("")
for (j in 1:n) {
  f1 <- fibers.Left$indices.of.fibers[j, 1]
  f2 <- fibers.Left$indices.of.fibers[j, 2]
  x <- fibers.Left.coord[f1:f2, 1]
  y <- fibers.Left.coord[f1:f2, 2]
  z <- fibers.Left.coord[f1:f2, 3]
  leftFibers[[j]] <- list(voxels = 0, coords = cbind(x, y, z))
}

set.seed(2)

search.for.deepest.curves <- function(Fibers = leftFibers, nbtries = 200, 
                                      k = floor(n ^ 0.8), ptsPerCurve = 256) {
    deepest <- rep(NA, nbtries)
    n <- length(Fibers)
    leftFibersk <- list("")
    for (l in 1:nbtries) {
        subSamp <- sample(1:n, k)
        for (j in 1:k) leftFibersk[[j]] <- Fibers[[subSamp[j]]]
        leftFibersSmplk <- sample.curves(leftFibersk, ptsPerCurve = ptsPerCurve)
        dssLeft <- depth.curve.Tukey(leftFibersSmplk, leftFibersSmplk, 
                                     exactEst = FALSE)
        deepest[l] <- subSamp[which.max(dssLeft)]
    }
    return(deepest)
}

time1 <- system.time({deepest1 <- search.for.deepest.curves()})
print(time1)
save.image(file = "deepest.RData")
tbl1 <- table(deepest1)
deep.curves1 <- as.numeric(names(which(tbl1 == max(tbl1))))
print(sort(tbl1, decreasing = TRUE)[1:10])

time2 <- system.time({deepest2 <- search.for.deepest.curves()})
print(time2)
save.image(file = "deepest.RData")
tbl2 <- table(deepest2)
deep.curves2 <- as.numeric(names(which(tbl2 == max(tbl2))))
print(sort(tbl2, decreasing = TRUE)[1:10])

# The two calls above should give (we hope) the same deepest curve (i.e., 
# same first number appearing each time).

q()
